/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.2
  * @date     2022-09-16
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f435_437.h"
#include "at32f435_437_board.h"
#include "at32f435_437_clock.h"
#include "stdio.h"
#include "xmc_lcd.h"
#include "ov2640.h"
#include "ov5640.h"
#include "dvp.h"
#include "zbar_decode.h"
#include "string.h"
#include "usb_config.h"

/** @addtogroup AT32F435_application_note
  * @{
  */

/** @addtogroup 435_qr_decode_by_zbar qr_decode_by_zbar
  * @{
  */

uint16_t camera_id = 0;
uint16_t cnt = CAMERA_SIZE * CAMERA_SIZE;
u8 Picture_Buffer1[CAMERA_SIZE * CAMERA_SIZE];
u16 Picture_Buffer2[CAMERA_SIZE * CAMERA_SIZE];

u32 frm_count;
u32 zbar_count;
u32 zbar_s;

uint8_t lcd_display_info = 0;
uint8_t dvp_dma_flag = 0, xmc_dma_flag = 0;
extern otg_core_type otg_core_struct;

/* usart global struct define */
void usart_gpio_config(void);

#define  decode_buffer_size  128
uint8_t decode_result[decode_buffer_size];
uint8_t last_decode_result[decode_buffer_size];
uint8_t string_decode_times[4] = {0};
uint8_t string_decode_times_usb[5] = {"000\r\n"};
uint8_t string_title[] = {"\r\nThe times and result: "};

void extend_sram(void);
uint16_t camera_identify(void);
void buzz_config(void);
void qr_decode(void);
void ov2640_capture(void);
void ov5640_capture(void);

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
uint16_t point_color;
int main(void)
{
  lcd_struct = &lcd_dev_struct;
	
  system_clock_config();
  
  at32_board_init();
	uart_print_init(115200);
	
	frm_count = 0;
	zbar_count = 0;
	zbar_s = 0;

  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  buzz_config();
  
  /* hardware usart config: usart2 */
  usb_usart_config(linecoding);

  /* usb gpio config */
  usb_gpio_config();
	
	/* enable otgfs clock */
  crm_periph_clock_enable(OTG_CLOCK, TRUE);

  /* select usb 48m clcok source */
  usb_clock48m_select(USB_CLK_HEXT);

  /* enable otgfs irq */
  nvic_irq_enable(OTG_IRQ, 0, 0);

  /* init usb */
  usbd_init(&otg_core_struct,
            USB_FULL_SPEED_CORE_ID,
            USB_ID,
            &cdc_class_handler,
            &cdc_desc_handler);
	
  lcd_struct->lcd_init();
  point_color = WHITE;
  lcd_fill(0, 0, 240, 320, WHITE);
  
  camera_id = camera_identify();
  while((camera_id != OV5640_ID) && (camera_id != OV2640_PID))
  {
    lcd_string_show(30, 100, 240, 16, 16, (uint8_t *)"Camer ERR");
    delay_ms(200);
    lcd_fill(30, 130, 239, 170, WHITE);
    delay_ms(200);
    at32_led_toggle(LED2);
  }
  
  if (camera_id == OV5640_ID)
  {
    while(ov5640_init() != SUCCESS)
    {
      lcd_string_show(30, 240, 240, 16, 16, (uint8_t *)"OV5640 ERR");
      delay_ms(200);
      lcd_fill(30, 130, 239, 170, WHITE);
      delay_ms(200);
      at32_led_toggle(LED2);
    }
  }
  else
  {
    while(ov2640_init() != SUCCESS)
    {
      lcd_string_show(30, 130, 240, 16, 16, (uint8_t *)"OV2640 ERR");
      delay_ms(200);
      lcd_fill(30, 130, 239, 170, WHITE);
      delay_ms(200);
      at32_led_toggle(LED2);
    }
  }

  lcd_fill(0, 0, 240, 320, WHITE);
  lcd_string_show(30, 100, 200, 16, 16, (uint8_t *)"Camera OK");
  lcd_string_show(10, 240, 220, 16, 16, (uint8_t*)"The times and result :");
  if (camera_id == OV5640_ID)
  {
    ov5640_capture();
  }
  else
  {
    ov2640_capture();
  }
  
  while(1)
  {    
    #ifdef LCD_DISPLAY
    while(xmc_dma_flag == 0){}
    xmc_dma_flag = 0;
    lcd_struct->lcd_setblock(0, 0, CAMERA_SIZE-1, CAMERA_SIZE-1);
    EDMA_STREAM1->dtcnt = cnt;
    edma_stream_enable(EDMA_STREAM1, TRUE);
      
    while(dvp_dma_flag == 0){}
    dvp_dma_flag = 0;
    ConvertY8toRGB565(Picture_Buffer1, Picture_Buffer2);
    EDMA_STREAM4->dtcnt = cnt/4;
    edma_stream_enable(EDMA_STREAM4, TRUE);
    dvp_capture_enable(TRUE);
    #endif

    qr_decode();
    if(at32_button_press() == USER_BUTTON)
    {
      lcd_display_info ^= 1;
    }
  }
}

uint16_t camera_identify(void)
{
  uint8_t temp = 0;
  uint16_t reg;
  gpio_init_type gpio_initure;

  crm_periph_clock_enable(CRM_GPIOC_PERIPH_CLOCK, TRUE);
  gpio_default_para_init(&gpio_initure);
  /* DVP_RST Pin-PD6 */
  gpio_initure.gpio_pins = GPIO_PINS_3 | GPIO_PINS_1;
  gpio_initure.gpio_mode = GPIO_MODE_OUTPUT;
  gpio_initure.gpio_out_type = GPIO_OUTPUT_PUSH_PULL;
  gpio_initure.gpio_pull = GPIO_PULL_UP;
  gpio_initure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(GPIOC, &gpio_initure);

  OV5640_RST_LOW;
  delay_ms(100);
  OV5640_RST_HIGH;
  delay_ms(20);
  ov5640_i2c_init();
  delay_ms(5);
  ov5640_reg_read(OV5640_CHIPIDH, &temp);
  reg  = (uint16_t)temp << 8;
  ov5640_reg_read(OV5640_CHIPIDL, &temp);
  reg |= temp;
  
  if(reg == OV5640_ID)
  {
    return reg;
  }
  
  OV2640_RST_LOW;		//POWER ON;
	delay_ms(10);
  OV2640_PWRON_LOW;
	delay_ms(10);
  OV2640_RST_HIGH;
	delay_ms(10);
  ov2640_i2c_init();
	ov2640_reg_write(OV2640_DSP_RA_DLMT, 0x01);
 	ov2640_reg_write(OV2640_SENSOR_COM7, 0x80);
	delay_ms(50); 
	ov2640_reg_read(OV2640_SENSOR_MIDH, &temp);
	reg  = (uint16_t)temp << 8;
	ov2640_reg_read(OV2640_SENSOR_MIDL, &temp);
  reg |= temp;
	if(reg!=OV2640_MID)
	{
		return 0;
	}
	ov2640_reg_read(OV2640_SENSOR_PIDH, &temp);
	reg  = (uint16_t)temp << 8;
	ov2640_reg_read(OV2640_SENSOR_PIDL, &temp);	
  reg |= temp;
	if(reg==OV2640_PID)
	{
		return reg;
	}
  
  return 0;
}

void qr_decode(void)
{
  static uint8_t send_packet = 0, decode_times = 0;
  uint32_t timeout;
	int zbar_find = 0;
  
  zbar_find = zbar_decode((void* )Picture_Buffer1,CAMERA_SIZE,CAMERA_SIZE, decode_result);
  if(zbar_find)
  {    
    if(lcd_display_info)
    {
      tmr_counter_enable(TMR3, TRUE);
    }
    if(strcmp((const char *)last_decode_result, (const char *)decode_result) != 0)
    {
      lcd_fill(10, 256, 240, 320, WHITE);
      decode_times = 0;
      lcd_string_show(10, 256, 220, 64, 16, (uint8_t *)decode_result);
      memset(last_decode_result,0,sizeof(last_decode_result));
      memcpy(last_decode_result, decode_result, strlen((const char *)decode_result));
    }
    decode_times++;
    string_decode_times[0] = decode_times/100+0x30;
    string_decode_times[1] = decode_times%100/10+0x30;
    string_decode_times[2] = decode_times%10+0x30;
    lcd_string_show(190, 240, 100, 16, 16, (uint8_t*)string_decode_times);
    memcpy(string_decode_times_usb, string_decode_times, 3);
    if(usbd_connect_state_get(&otg_core_struct.dev) == USB_CONN_STATE_CONFIGURED)
    {
        timeout = 50000;
        do
        {
          if(usb_vcp_send_data(&otg_core_struct.dev, string_title, 24) == SUCCESS)
          {
            send_packet ++;
            break;
          }
        }while(timeout --);
        timeout = 50000;
        do
        {
          if(usb_vcp_send_data(&otg_core_struct.dev, string_decode_times_usb, 5) == SUCCESS)
          {
            send_packet ++;
            break;
          }
        }while(timeout --);
        timeout = 50000;
        do
        {
          if(usb_vcp_send_data(&otg_core_struct.dev, decode_result, strlen((const char *)decode_result)) == SUCCESS)
          {
            send_packet ++;
            break;
          }
        }while(timeout --);
        memset(decode_result,0,sizeof(decode_result));
    }
  }
}

void ov2640_capture(void)
{
  ov2640_rgb565_mode();
  ov2640_light_mode(0);
  ov2640_color_saturation(3);
  ov2640_brightness(4);
  ov2640_contrast(3);

  dvp_config(camera_id);
  
	dvp_dma_init((uint32_t)&Picture_Buffer1, cnt/4);
  #ifdef LCD_DISPLAY
  xmc_dma_init((uint32_t)&Picture_Buffer2, cnt);
  #endif
  
  delay_ms(100);
  ov2640_imagewin_set(100,0,600,600);
  ov2640_outsize_set(CAMERA_SIZE, CAMERA_SIZE);
  delay_ms(100);
  dvp_start();
}

void ov5640_capture(void)
{
  uint8_t reg_val = 0;
  uint8_t fac;

  ov5640_rgb565_mode();
  ov5640_focus_init();
  ov5640_light_mode(0);
  ov5640_color_saturation(3);
  ov5640_brightness(4);
  ov5640_contrast(3);
  ov5640_sharpness(33);
  ov5640_focus_constant();

  dvp_config(camera_id);

	dvp_dma_init((uint32_t)&Picture_Buffer1, cnt/4);
  
  #ifdef LCD_DISPLAY
  xmc_dma_init((uint32_t)&Picture_Buffer2, cnt);
  #endif

  delay_ms(100);
  fac = 1200/CAMERA_SIZE;
  ov5640_outsize_set((1600-fac*CAMERA_SIZE)/2,(1200-fac*CAMERA_SIZE)/2, CAMERA_SIZE, CAMERA_SIZE);
  ov5640_reg_read(0x4730, &reg_val);
  dvp_start();
}

void buzz_config(void)
{
  gpio_init_type gpio_initure;
  tmr_output_config_type tmr_oc_init_structure;
  crm_clocks_freq_type crm_clocks_freq_struct = {0};
  
  crm_periph_clock_enable(CRM_GPIOE_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_TMR3_PERIPH_CLOCK, TRUE);
  
  gpio_initure.gpio_pins = GPIO_PINS_3;
  gpio_initure.gpio_mode = GPIO_MODE_MUX;
  gpio_initure.gpio_out_type = GPIO_OUTPUT_PUSH_PULL;
  gpio_initure.gpio_pull = GPIO_PULL_NONE;
  gpio_initure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(GPIOE, &gpio_initure);
  gpio_pin_mux_config(GPIOE, GPIO_PINS_SOURCE3, GPIO_MUX_2);

  crm_clocks_freq_get(&crm_clocks_freq_struct);
  /* time base configuration */
  tmr_base_init(TMR3, 199, (crm_clocks_freq_struct.apb2_freq * 2 / 10000) - 1);
  tmr_cnt_dir_set(TMR3, TMR_COUNT_UP);
  tmr_one_cycle_mode_enable(TMR3, TRUE);
  
  tmr_output_default_para_init(&tmr_oc_init_structure);
  tmr_oc_init_structure.oc_mode = TMR_OUTPUT_CONTROL_PWM_MODE_B;
  tmr_oc_init_structure.oc_idle_state = FALSE;
  tmr_oc_init_structure.oc_polarity = TMR_OUTPUT_ACTIVE_HIGH; 
  tmr_oc_init_structure.oc_output_state = TRUE; 
  tmr_output_channel_config(TMR3, TMR_SELECT_CHANNEL_1, &tmr_oc_init_structure);
  tmr_channel_value_set(TMR3, TMR_SELECT_CHANNEL_1, 1);
}

/**
  * @brief  to extend sram size
  * @param  none
  * @retval none
  */
void extend_sram(void)
{
  /* check if ram has been set to expectant size, if not, change eopb0 */
  if(((USD->eopb0) & 0x07) != FLASH_EOPB0_SRAM_384K)
  {
    flash_unlock();
    /* erase user system data bytes */
    flash_user_system_data_erase();
    
    /* change sram size */
    flash_eopb0_config(FLASH_EOPB0_SRAM_384K);
    
    /* system reset */
    nvic_system_reset();
  }
}

/**
  * @}
  */

/**
  * @}
  */
